/*
 * Copyright (c) 2010, 2011 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: dkimcnfdef.h,v 1.30 2013/12/13 02:08:19 ca Exp $
 */

#ifndef SM_DKIMCNFDEF_H
#define SM_DKIMCNFDEF_H 1

#include "sm/generic.h"
#if MTA_USE_DKIM
#include "sm/sm-conf-byte.h"
#include "sm/dkimdef.h"

#if SM_DKIMCNFDEF
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN
sm_conf_definition_T const
dkim_sign_methods[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC,
	"rsa-sha256",			sm_conf_type_choice_value,
	DKIM_SM_RSA_SHA256,
	0, NULL, 0, NULL, NULL, NULL,
	"RSA SHA256"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC,
	"rsa-sha1",			sm_conf_type_choice_value,
	DKIM_SM_RSA_SHA1,
	0, NULL, 0, NULL, NULL, NULL,
	"RSA SHA1"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;

EXTERN
sm_conf_definition_T const
dkim_canon_methods[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC,
	"simple",			sm_conf_type_choice_value,
	DKIM_CANON_SIMPLE,
	0, NULL, 0, NULL, NULL, NULL,
	"simple canonicalization"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC,
	"relaxed",			sm_conf_type_choice_value,
	DKIM_CANON_RELAXED,
	0, NULL, 0, NULL, NULL, NULL,
	"relaxed canonicalization"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;


EXTERN
sm_conf_definition_T
dkim_s_def[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "name",		sm_conf_type_section_title,
	offsetof(dkim_s_T, dks_name),	0,
	NULL,	0,
	NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "keyfile",	sm_conf_type_string,
	offsetof(dkim_s_T, dks_keyfile),	0,
	NULL,			SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_STRICTLY_REQUIRED,
	NULL,	NULL,	NULL,
	"private key file"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "selector",	sm_conf_type_string,
	offsetof(dkim_s_T, dks_selector),	0,
	NULL,			SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_STRICTLY_REQUIRED,
	NULL,	NULL,	NULL,
	"selector name"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "domain",	sm_conf_type_string,
	offsetof(dkim_s_T, dks_domain),	0,
	NULL,			SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_STRICTLY_REQUIRED,
	NULL,	NULL,	NULL,
	"domain"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "from",	sm_conf_type_string,
	offsetof(dkim_s_T, dks_from),	0,
	NULL,			SM_CONF_FLAG_KEEP_DEFAULT,
	NULL,	NULL,	NULL,
	"only apply this to mails with matching From: address"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;

#if 0
/*
 *  unfortunately this can't be used because the lengths already have
 *  sm_conf_byte_suffixes, which uses the same field as constraints.
 */
EXTERN
sm_conf_definition_T const
dkim_hdrtlen_constraints[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_minimum, 256,
	0, NULL, 0, NULL, NULL, NULL, NULL },
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_maximum, 1 UNIT_GBYTES,
	0, NULL, 0, NULL, NULL, NULL, NULL },
CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;

EXTERN
sm_conf_definition_T const
dkim_hdrvlen_constraints[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_minimum, 128,
	0, NULL, 0, NULL, NULL, NULL, NULL },
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_maximum, 1 UNIT_MBYTES,
	0, NULL, 0, NULL, NULL, NULL, NULL },
CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;
#endif


EXTERN
sm_conf_definition_T
dkim_s_definition[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "",		sm_conf_type_section,
	offsetof(dkim_cnf_T, dkimcnf_s), sizeof(dkim_s_T),
	NULL, 0,
	dkim_s_def,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "n", sm_conf_type_array_n,
	offsetof(dkim_cnf_T, dkimcnf_n), sizeof(uint),
	NULL, 0, NULL, NULL, NULL,
	"number of DKIM signing definitions (implicit)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;

sm_conf_definition_check_T dkim_hdrtlencb;
sm_conf_definition_check_T dkim_hdrvlencb;

EXTERN
sm_conf_definition_T
dkim_defs[]
#if SM_DKIMCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "signature_method",		sm_conf_type_choice,
	offsetof(dkim_cnf_T, dkimcnf_signmethod),	sizeof(uint),
	NULL, 0,
	dkim_sign_methods,
	NULL,	NULL,
	"signature method"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "body_canonicalization",		sm_conf_type_choice,
	offsetof(dkim_cnf_T, dkimcnf_bcanon),	sizeof(uint),
	NULL, 0,
	dkim_canon_methods,
	NULL,	NULL,
	"canonicalization algorithm for mail body"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "header_canonicalization",		sm_conf_type_choice,
	offsetof(dkim_cnf_T, dkimcnf_hcanon),	sizeof(uint),
	NULL, 0,
	dkim_canon_methods,
	NULL,	NULL,
	"canonicalization algorithm for mail header"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "skip_header",		sm_conf_type_argv,
	offsetof(dkim_cnf_T, dkimcnf_skip),	DKIM_HDR_LIST_MAX,	NULL,
	0,	NULL,	NULL,	NULL,
	"list of headers to skip during signing"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "sign_header",		sm_conf_type_argv,
	offsetof(dkim_cnf_T, dkimcnf_sign),	DKIM_HDR_LIST_MAX,	NULL,
	0,	NULL,	NULL,	NULL,
	"list of headers to sign"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "ignore_errors",		sm_conf_type_bool,
	offsetof(dkim_cnf_T, dkimcnf_ignerr),	sizeof(bool),
	NULL, 0, NULL, NULL,	NULL,
	"ignore signing errors?"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "always_sign_headers",		sm_conf_type_argv,
	offsetof(dkim_cnf_T, dkimcnf_add),	DKIM_HDR_LIST_MAX,	NULL,
	0,	NULL,	NULL,	NULL,
	"list of headers to add to signature even if they are not in the message"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "total_header_len",	sm_conf_type_u32,
	offsetof(dkim_cnf_T, dkimcnf_hdrtlen),	sizeof(uint32_t),
	NULL /* SM_XSTR(SM_TOTAL_HDR_LEN) */,	SM_CONF_FLAG_KEEP_DEFAULT,
	sm_conf_byte_suffixes,	dkim_hdrtlencb,	NULL,
	"total header length allowed"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0)
	},

{ SM_CONF_DEF_MAGIC, "header_val_len_max",	sm_conf_type_u32,
	offsetof(dkim_cnf_T, dkimcnf_hdrvlen),	sizeof(uint32_t),
	NULL /* SM_XSTR(SM_HDR_LEN_MAX) */,	SM_CONF_FLAG_KEEP_DEFAULT,
	sm_conf_byte_suffixes,	dkim_hdrvlencb,	NULL,
	"maximum length of a single header value"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0)
	},

{ SM_CONF_DEF_MAGIC, "sign",		sm_conf_type_array,
	0,	/* offset */
	0,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE,
	dkim_s_definition,
	NULL,  NULL,
	"DKIM selector declarations"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_DKIMCNFDEF */
;

#undef EXTERN
#endif /* MTA_USE_DKIM */

#endif /* SM_DKIMCNFDEF_H */
