#!/bin/sh
# $Id: common.sh,v 1.21 2013/11/26 03:09:16 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Initialize some variables used by many smar test programs
# ------------------------------------------------------------

test -s stop && exit 1

if [ X"${TPRG}" != "X" ]
then
	T=`echo ${TPRG} | sed -e 's/ .*//'`
	if [ ! -x ${T} ]
	then
		echo "$0: ERROR: missing test program ${TPRG}"
		exit 1
	fi
fi

# see include/sm/mta.h

MTA_LOCALHOST=${MTA_LOCALHOST:-"127.0.0.1"}
MTA_LMTPIPV4D=${MTA_LMTPIPV4D:-"127.0.0.255"}
MTA_LMTPIPV4H=${MTA_LMTPIPV4H:-"7f0000ff"}
# export MTA_LOCALHOST MTA_LMTPIPV4D MTA_LMTPIPV4H

SMAROPTS=${MTA_SMAROPTS}

BASEOUT=`basename $0 .sh | sed -e s'/t-//'`
OUT=${BASEOUT}.out
OK=${BASEOUT}.ok
IP=${BASEOUT}.ip
TMP=${BASEOUT}.tmp

# Note: reference to t-hostname! XREF
HOSTNAME=`../libmta/t-hostname -l -V 2>/dev/null`
if test $? -ne 0
then
  echo "t-hostname failed"
  exit 1
fi
if test X"${HOSTNAME}" = X
then
  echo "t-hostname returned empty name"
  exit 1
fi
if echo X"${HOSTNAME}" | grep '\.' >/dev/null
then
  :
else
  echo "t-hostname returned bogus name ${HOSTNAME} (missing dot)"
  exit 1
fi

ALI=aliases
ALIMAP=${ALI}.db
ACC=access
ACCMAP=${ACC}.db
MT=mt
MTMAP=${MT}
# temporary hack
MTMAPDB=${MT}.db

SKIP=""
VERBOSE=false

ALOG=a.log

MM=../libsmmap/t-mm

SOCK=qmsmar/qmsmar

PIDS=pids
ERRS=0

MAPC=mapc
MMMAPC=mmmapc

SHELL=/bin/sh
test -x /bin/ksh && SHELL=/bin/ksh

CAT=cat

# XREF: SMTP_R_CONT
SMAR_DFLT_RES=399

chkcto()
{
  CTO=$1
  shift

  if ./smar -VVV | grep "$CTO" >/dev/null
  then
    :
  else
    echo "SKIPPED: $0: smar not compiled with $CTO $@"
    exit 0
  fi
}

chkctoprg()
{
  tprg=$1
  CTO=$2
  shift 2

  if ${tprg} -C | grep "$CTO" >/dev/null
  then
    :
  else
    echo "SKIPPED: $0: ${tprg} not compiled with $CTO $@"
    exit 0
  fi
}


GETNS=${SD}/../misc/getns.sh
if test -x ${GETNS}
then
  . ${GETNS}
else
NS=`grep '^nameserver' /etc/resolv.conf | head -n1 | sed -e 's/nameserver[	 ]*\([0-9][0-9\.]*\).*$/\1/'`
fi
