#!/bin/sh
# $Id: t-smar-mx-1.sh,v 1.4 2013/11/19 04:59:18 ca Exp $
#
# ------------------------------------------------------------
# Description:
# Script to test MX/AAAA expansion for RFC2821 addresses.
# Uses smar, t-smar.sh, DNS
# Note: this program relies on the meta1.org DNS server.
# ------------------------------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0"
  exit 0
fi

if test X"${MTA_NO_DNS_HACK_TEST}" != "X"
then
  echo "SKIPPED: $0"
  exit 0
fi

TPRG=./t-smar-1
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh
chkctoprg ${TPRG} MTA_NETINET6

rm -f ${SOCK} ${PIDS}

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# start SMAR
# temp.HACK: -6 for smar to use IPv6
${SHELL} ${SD}/startsmar.sh -o -6 ${SMAROPTS} || exit 1

TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
A='<a2@ipmx0.meta1.org>'
${VALGRIND} ${TPRG} -M -VVVV ${A} > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=fec0:0:0:0:2fd:87fe:1e72:1d4f [fec00000]
IP[1]=10.0.0.1 [a000001]
EOF
if diff ${OK} ${IP}
then
  rm -f ${OUT} ${IP} ${OK}
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

fi


TEST=2
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
# ----------------------------------------
A='<a3@ipmx1.meta1.org>'
${VALGRIND} ${TPRG} -M -VVVV ${A} > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=10.0.0.1 [a000001]
IP[1]=fec0:0:0:0:2fd:87fe:1e72:1d4f [fec00000]
EOF
if diff ${OK} ${IP}
then
  rm -f ${OUT} ${IP} ${OK}
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

fi

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi

exit ${ERRS}
